------------------------------------------------------------------------------
--  This file is a part of the LPP VHDL IP LIBRARY
--  Copyright (C) 2009 - 2010, Laboratory of Plasmas Physic - CNRS
--
--  This program is free software; you can redistribute it and/or modify
--  it under the terms of the GNU General Public License as published by
--  the Free Software Foundation; either version 3 of the License, or
--  (at your option) any later version.
--
--  This program is distributed in the hope that it will be useful,
--  but WITHOUT ANY WARRANTY; without even the implied warranty of
--  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
--  GNU General Public License for more details.
--
--  You should have received a copy of the GNU General Public License
--  along with this program; if not, write to the Free Software
--  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA 
-------------------------------------------------------------------------------
--                    Author : Martin Morlot
--                   Mail : martin.morlot@lpp.polytechnique.fr
-------------------------------------------------------------------------------
LIBRARY IEEE;
USE IEEE.numeric_std.ALL;
USE IEEE.std_logic_1164.ALL;

--! Programme permetant de complmenter ou non les entres de l'ALU, et ainsi de travailler avec des nombres ngatifs

ENTITY TwoComplementer_LPP_JCP IS
  GENERIC(
    Input_SZ : INTEGER := 16);
  PORT(
    clk     : IN  STD_LOGIC;            --! Horloge du composant
    reset   : IN  STD_LOGIC;            --! Reset general du composant
    clr     : IN  STD_LOGIC;            --! Un reset spcifique au programme
    TwoComp : IN  STD_LOGIC;  --! Autorise l'utilisation du complment
    OP      : IN  STD_LOGIC_VECTOR(Input_SZ-1 DOWNTO 0);  --! Oprande d'entre
    RES     : OUT STD_LOGIC_VECTOR(Input_SZ-1 DOWNTO 0)  --! Rsultat, oprande complment ou non
    );
END TwoComplementer_LPP_JCP;


ARCHITECTURE ar_TwoComplementer OF TwoComplementer_LPP_JCP IS

  SIGNAL REG       : STD_LOGIC_VECTOR(Input_SZ-1 DOWNTO 0);
  SIGNAL OPinteger : INTEGER;
  SIGNAL RESCOMP   : STD_LOGIC_VECTOR(Input_SZ-1 DOWNTO 0);

BEGIN

  RES       <= REG;
  OPinteger <= to_integer(SIGNED(OP));
  RESCOMP   <= STD_LOGIC_VECTOR(to_signed(-OPinteger, Input_SZ));

  PROCESS(clk, reset)
  BEGIN

    IF(reset = '0')then
      REG <= (OTHERS => '0');
    ELSIF(clk'EVENT AND clk = '1')then
      
      IF(clr = '1')then
        REG <= (OTHERS => '0');
      ELSIF(TwoComp = '1')then
        REG <= RESCOMP;
      ELSE
        REG <= OP;
      END IF;

    END IF;

  END PROCESS;
END ar_TwoComplementer;
